import org.hyperic.hq.authz.shared.PermissionManagerFactory
import org.hyperic.hq.authz.shared.PermissionException
import org.hyperic.hq.authz.shared.AuthzConstants
import org.hyperic.hq.hqu.rendit.BaseController

class SummaryController
    extends BaseController {

    /**
     * Get the custom properties for a resource
     * key = Cprop description
     * value = Cprop value
     */
    private getCustomProperties(resource) {
        def config = resource.config
        def props = [:]

        config.each { k, v ->
            if (v.type.equals("cprop")) {
                props[v.description] = v.value
            }
        }
        props
    }
    
    private boolean hasPlatformPermission(instanceId) {
        try {
            PermissionManagerFactory.getInstance().check(user.getId(), 
                                                     AuthzConstants.platformResType, 
                                                     instanceId,
                                                     AuthzConstants.platformOpViewPlatform)
        } catch(PermissionException e) {
            return false
        }
        
        return true
    }
    
    private boolean hasServerPermission(instanceId) {
        try {
            PermissionManagerFactory.getInstance().check(user.getId(), 
                                                     AuthzConstants.serverResType, 
                                                     instanceId,
                                                     AuthzConstants.serverOpViewServer)
        } catch(PermissionException e) {
            return false
        }
        
        return true
    }

    def index(params) {
        def id = params.getOne('id')?.toInteger()

        if (!id) {  
            render(inline: "No resource selected")
            
            return
        }

        def resource = resourceHelper.findById(id)
        def ancestors = resourceHelper.findAncestorsByVirtualRelation(resource)
        def host = null, hostProps = null, canViewHost = false,
            vm = null, vmProps = null, associatedPlatform = null, canViewVm = false,
            server = null, serverProps = null, canViewServer = false

        if (resource.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereHost) {
            host = resource
            hostProps = getCustomProperties(resource)
            canViewHost = hasPlatformPermission(resource.getInstanceId())
        } else if (resource.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereVm) {
            vm = resource
            vmProps = getCustomProperties(resource)
            canViewVm = hasPlatformPermission(resource.getInstanceId())
            
            def children = resourceHelper.findChildResourcesByVirtualRelation(resource)
            def platform = children.find({ res -> res.resourceType.id == AuthzConstants.authzPlatform })
                
            if (platform && hasPlatformPermission(platform.getInstanceId())) {
                associatedPlatform = platform
            }
        } else {
            server = resource
            serverProps = getCustomProperties(resource)
            canViewServer = hasServerPermission(resource.getInstanceId())
        }
            
        ancestors.each { res ->
            if (res.prototype.name == AuthzConstants.serverPrototypeVmwareVcenter) {
                // TODO
            } else if (res.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereHost) {
                host = res
                hostProps = getCustomProperties(res)
                canViewHost = hasPlatformPermission(res.getInstanceId())
            } else if (res.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereVm) {
                vm = res
                vmProps = getCustomProperties(res)
                canViewVm = hasPlatformPermission(res.getInstanceId())
                
                def children = resourceHelper.findChildResourcesByVirtualRelation(resource)
                def platform = children.find({ r -> r.resourceType.id == AuthzConstants.authzPlatform })
                
                if (platform && hasPlatformPermission(platform.getInstanceId())) {
                    associatedPlatform = platform
                }
            }
        }
        
        render(locals:[ host : host,
                        hostProps : hostProps,
                        canViewHost : canViewHost,
                        vm : vm,
                        vmProps : vmProps,
                        canViewVm : canViewVm,
                        associatedPlatform: associatedPlatform,
                        resource : server,
                        resourceProps : serverProps,
                        canViewResource : canViewServer ])
    }
}