import org.apache.commons.math.stat.regression.SimpleRegression

class XtrapMethodSimpleRegression implements XtrapMethod {

    def getName() {
        "Simple Regression"
    }

    def extrapolate(List data, long range) {
        SimpleRegression r = new SimpleRegression();
        data.each { r.addData(it.timestamp, it.value) }

        def ret = []
        long interval = range / 100 // Always return 100 future points
        def ts = data[-1].timestamp + interval
        while (ts <= data[-1].timestamp + range) {
            def val = new XtrapData()
            val.timestamp = ts
            val.value = r.predict(ts)
            ret += val
            ts += interval
        }
        ret
    }

    /*
     * Get the confidence of the estimation for a list of measurements.  The
     * confidence value is the percentage variance from the function in which
     * 95% of the known data points fall.
     */
    def getConfidence(List measurements, long range) {
        long now = System.currentTimeMillis()
        def ret = [:]
        measurements.each { m ->
            List data = m.getData(now - range, now)
            SimpleRegression r = new SimpleRegression()
            data.each { r.addData(it.timestamp, it.value) }

            // The confidence calculated here will be the % from the slope line
            // in which 95% of all measurements fall.
            def val = (((r.getSlopeConfidenceInterval().abs() +
                         r.getSlope().abs())/ r.getSlope().abs()) - 1) * 100
            def confidence = new XtrapConfidence()
            confidence.value = val
            if (val <= 5) {
                confidence.displayValue = "Excellent"
            } else if (val > 5 && val <= 10) {
                confidence.displayValue = "Good"
            } else if (val > 10 && val <= 30) {
                confidence.displayValue = "Average"
            } else if (val > 30 && val < 100) {
                confidence.displayValue = "Questionable"
            } else {
                confidence.displayValue = "Poor"
            }
            ret.put(m.id, confidence)
        }
        ret        
    }
}
