package org.hyperic.hq.hqu.rendit.metaclass

import org.hyperic.hq.authz.server.session.ResourceGroup
import org.hyperic.hq.authz.server.session.Resource
import org.hyperic.hq.authz.server.session.ResourceManagerEJBImpl as RsrcMan
import org.hyperic.hq.authz.server.session.AuthzSubject
import org.hyperic.hq.appdef.shared.AppdefEntityConstants
import org.hyperic.hq.authz.server.session.ResourceGroupManagerEJBImpl as GroupMan
import org.hyperic.hq.authz.shared.AuthzConstants
import org.hyperic.hq.events.shared.MaintenanceEventManagerInterface
import org.hyperic.hq.authz.shared.PermissionManagerFactory
import org.hyperic.hq.events.MaintenanceEvent

class ResourceGroupCategory {
    private static groupMan = GroupMan.one
    private static rsrcMan  = RsrcMan.one

    private static MaintenanceEventManagerInterface maintMan =
        PermissionManagerFactory.getInstance().getMaintenanceEventManager();

    static String urlFor(ResourceGroup r, String context) {
        "/Resource.do?eid=${AppdefEntityConstants.APPDEF_TYPE_GROUP}:${r.id}"
    }
    
    static void setResources(ResourceGroup group, AuthzSubject user,
                             Collection resources)
    {
        groupMan.setResources(user, group, resources)
    }

    static Collection getResources(ResourceGroup group) {
        groupMan.getMembers(group)
    }
    
    /**
     * Returns true if the ResourceGroup is compatible (i.e. only contains
     * resources of a homogenous type)
     */
    static boolean isCompatible(ResourceGroup g) {
        g.groupType == AppdefEntityConstants.APPDEF_TYPE_GROUP_COMPAT_PS ||
        g.groupType == AppdefEntityConstants.APPDEF_TYPE_GROUP_COMPAT_SVC
    }
    
    /**
     * Gets the Resource prototype that the group is compatible wth.
     */
    static Resource getCompatibleType(ResourceGroup g) {
        if (!isCompatible(g)) {
            throw new IllegalArgumentException("Attempted to get compatible " +
                                               "type on non-compatible group")
        }
        
        def resourceTypeId
        if (g.groupEntType == AppdefEntityConstants.APPDEF_TYPE_PLATFORM) {
            resourceTypeId = AuthzConstants.authzPlatformProto
        } else if (g.groupEntType == AppdefEntityConstants.APPDEF_TYPE_SERVER) {
            resourceTypeId = AuthzConstants.authzServerProto
        } else if (g.groupEntType == AppdefEntityConstants.APPDEF_TYPE_SERVICE) {
            resourceTypeId = AuthzConstants.authzServiceProto
        } else {
            assert "Unknown group ent type ${g.groupEntType} for compat groups"
        }
            
        rsrcMan.findResourceByInstanceId(resourceTypeId, g.groupEntResType)
    }
    
    static void updateGroupType(ResourceGroup g, AuthzSubject subject,
                                int groupType, int groupEntType,
                                int groupEntResType) {
        groupMan.updateGroupType(subject, g, groupType, groupEntType,
                                 groupEntResType)
    }
    
    static void updateGroup(ResourceGroup g, AuthzSubject subject,
                            String name, String description, String location) {
        groupMan.updateGroup(subject, g, name, description, location)
    }

    static void remove(ResourceGroup g, AuthzSubject subject) {
        groupMan.removeResourceGroup(subject, g)
    }

    static MaintenanceEvent scheduleMaintenance(ResourceGroup g, AuthzSubject subject,
                                                long start, long end) {
        MaintenanceEvent e = new MaintenanceEvent(g.getId());
        e.setStartTime(start)
        e.setEndTime(end)
        maintMan.schedule(subject, e)
    }

    static void unscheduleMaintenance(ResourceGroup g, AuthzSubject subject) {
        MaintenanceEvent e = new MaintenanceEvent(g.getId());
        maintMan.unschedule(subject, e)
    }

    static MaintenanceEvent getMaintenanceEvent(ResourceGroup g,
                                                AuthzSubject subject) {
        maintMan.getMaintenanceEvent(subject, g.getId())
    }
}
